"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LogService = void 0;
/*
 * Copyright (c) 2021, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const core_1 = require("@salesforce/core");
const kit_1 = require("@salesforce/kit");
const constants_1 = require("./constants");
const path = require("path");
const i18n_1 = require("../i18n");
const utils_1 = require("../utils");
const traceFlags_1 = require("../utils/traceFlags");
class LogService {
    constructor(connection) {
        this.connection = connection;
    }
    async getLogIds(options) {
        if (!(typeof options.logId === 'string' ||
            typeof options.numberOfLogs === 'number')) {
            throw new Error(i18n_1.nls.localize('missingInfoLogError'));
        }
        if (typeof options.numberOfLogs === 'number') {
            const logIdRecordList = await this.getLogRecords(options.numberOfLogs);
            return logIdRecordList.map((logRecord) => logRecord.Id);
        }
        return [options.logId];
    }
    // TODO: readableStream cannot be used until updates are made in jsforce and sfdx-core
    async getLogs(options) {
        const logIdList = await this.getLogIds(options);
        const logPaths = [];
        const connectionRequests = logIdList.map(async (id) => {
            const url = `${this.connection.tooling._baseUrl()}/sobjects/ApexLog/${id}/Body`;
            const logRecord = await this.toolingRequest(url);
            if (options.outputDir) {
                const logPath = path.join(options.outputDir, `${id}.log`);
                logPaths.push(logPath);
                (0, utils_1.createFile)(logPath, logRecord);
            }
            return String(logRecord);
        });
        const logs = await Promise.all(connectionRequests);
        if (logPaths.length > 0) {
            const logMap = [];
            for (let i = 0; i < logs.length; i++) {
                logMap.push({ log: logs[i], logPath: logPaths[i] });
            }
            return logMap;
        }
        return logs.map((log) => {
            return { log };
        });
    }
    async getLogById(logId) {
        const baseUrl = this.connection.tooling._baseUrl();
        const url = `${baseUrl}/sobjects/ApexLog/${logId}/Body`;
        const response = (await this.connection.tooling.request(url));
        return { log: response.toString() || '' };
    }
    async getLogRecords(numberOfLogs) {
        let apexLogQuery = `
        SELECT Id, Application, DurationMilliseconds, Location, LogLength, LogUser.Name,
          Operation, Request, StartTime, Status
        FROM ApexLog
        ORDER BY StartTime DESC
      `;
        if (typeof numberOfLogs === 'number') {
            if (numberOfLogs <= 0) {
                throw new Error(i18n_1.nls.localize('numLogsError'));
            }
            numberOfLogs = Math.min(numberOfLogs, constants_1.MAX_NUM_LOGS);
            apexLogQuery += ` LIMIT ${numberOfLogs}`;
        }
        const response = (await this.connection.tooling.query(apexLogQuery));
        return response.records;
    }
    async tail(org, tailer) {
        this.logger = await core_1.Logger.child('apexLogApi', { tag: 'tail' });
        this.logTailer = tailer;
        const stream = await this.createStreamingClient(org);
        this.logger.debug(i18n_1.nls.localize('startHandshake'));
        await stream.handshake();
        this.logger.debug(i18n_1.nls.localize('finishHandshake'));
        await stream.subscribe(async () => {
            this.logger.debug(i18n_1.nls.localize('subscribeStarted'));
        });
    }
    async createStreamingClient(org) {
        const options = new core_1.StreamingClient.DefaultOptions(org, constants_1.STREAMING_LOG_TOPIC, this.streamingCallback.bind(this));
        options.setSubscribeTimeout(kit_1.Duration.minutes(constants_1.LOG_TIMER_LENGTH_MINUTES));
        return await core_1.StreamingClient.create(options);
    }
    async logCallback(message) {
        if (message.sobject && message.sobject.Id) {
            const log = await this.getLogById(message.sobject.Id);
            if (log && this.logTailer) {
                this.logTailer(log.log);
            }
        }
    }
    streamingCallback(message) {
        if (message.errorName === constants_1.LISTENER_ABORTED_ERROR_NAME) {
            return { completed: true };
        }
        if (message.sobject?.Id) {
            this.logCallback(message);
        }
        return { completed: false };
    }
    async prepareTraceFlag(requestedDebugLevel) {
        const flags = new traceFlags_1.TraceFlags(this.connection);
        await flags.ensureTraceFlags(requestedDebugLevel);
    }
    async toolingRequest(url) {
        const log = (await this.connection.tooling.request(url));
        return log;
    }
}
exports.LogService = LogService;
//# sourceMappingURL=logService.js.map