/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
import { Messages } from '@salesforce/core';
import { QueryJobV2 } from 'jsforce/lib/api/bulk.js';
import { Flags, loglevel, optionalOrgFlagWithDeprecations, orgApiVersionFlagWithDeprecations, SfCommand, } from '@salesforce/sf-plugins-core';
import { resultFormatFlag } from '../../../flags.js';
import { displayResults, transformBulkResults } from '../../../queryUtils.js';
import { BulkQueryRequestCache } from '../../../bulkDataRequestCache.js';
Messages.importMessagesDirectoryFromMetaUrl(import.meta.url);
const reportMessages = Messages.loadMessages('@salesforce/plugin-data', 'bulk.report');
// needed by the flags loaded from the other command
const queryMessages = Messages.loadMessages('@salesforce/plugin-data', 'soql.query');
export class BulkQueryReport extends SfCommand {
    static summary = reportMessages.getMessage('summary');
    static description = reportMessages.getMessage('description');
    static examples = reportMessages.getMessages('examples');
    static aliases = ['force:data:soql:bulk:report'];
    static deprecateAliases = true;
    static flags = {
        'target-org': { ...optionalOrgFlagWithDeprecations, summary: queryMessages.getMessage('flags.targetOrg.summary') },
        'api-version': orgApiVersionFlagWithDeprecations,
        loglevel,
        'result-format': resultFormatFlag,
        'bulk-query-id': Flags.salesforceId({
            length: 18,
            char: 'i',
            startsWith: '750',
            summary: reportMessages.getMessage('flags.bulkQueryId.summary'),
            aliases: ['bulkqueryid'],
            deprecateAliases: true,
        }),
        'use-most-recent': Flags.boolean({
            summary: reportMessages.getMessage('flags.useMostRecent.summary'),
            exclusive: ['bulk-query-id'],
        }),
    };
    async run() {
        const { flags } = await this.parse(BulkQueryReport);
        const cache = await BulkQueryRequestCache.create();
        const resumeOptions = await cache.resolveResumeOptionsFromCache(flags['bulk-query-id'], flags['use-most-recent'], flags['target-org'], flags['api-version']);
        const job = new QueryJobV2({ ...resumeOptions.options, operation: 'query' });
        job.jobInfo = resumeOptions.jobInfo;
        const results = await job.getResults();
        const queryResult = transformBulkResults(results, '');
        if (!this.jsonEnabled()) {
            displayResults({ ...queryResult }, flags['result-format']);
        }
        if (queryResult.result.done) {
            await BulkQueryRequestCache.unset(resumeOptions.jobInfo.id);
        }
        return queryResult.result;
    }
}
//# sourceMappingURL=resume.js.map