/*
 * Copyright (c) 2021, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
/* eslint-disable class-methods-use-this */
import { EOL } from 'node:os';
import { writeFile, readFile } from 'node:fs/promises';
import { existsSync } from 'node:fs';
import { exec } from 'node:child_process';
import { Messages } from '@salesforce/core';
import { Env, parseJsonMap } from '@salesforce/kit';
import { generateTableChoices, Prompter, SfCommand, SfHook, Flags, } from '@salesforce/sf-plugins-core';
Messages.importMessagesDirectoryFromMetaUrl(import.meta.url);
const messages = Messages.loadMessages('@salesforce/plugin-deploy-retrieve', 'deploy');
export const DEPLOY_OPTIONS_FILE = 'deploy-options.json';
export default class Deploy extends SfCommand {
    static summary = messages.getMessage('summary');
    static description = messages.getMessage('description');
    static examples = messages.getMessages('examples');
    static enableJsonFlag = false;
    static state = 'deprecated';
    static hidden = true;
    static deprecationOptions = {
        version: '59.0',
        message: messages.getMessage('deprecation'),
    };
    static flags = {
        interactive: Flags.boolean({
            summary: messages.getMessage('flags.interactive.summary'),
        }),
    };
    async run() {
        process.setMaxListeners(new Env().getNumber('SF_MAX_EVENT_LISTENERS') ?? 1000);
        const { flags } = await this.parse(Deploy);
        flags.interactive = await this.isInteractive(flags.interactive);
        const options = await this.readOptions();
        this.log(messages.getMessage('AnalyzingProject'));
        if (!flags.interactive) {
            this.log(messages.getMessage('UsingOptionsFromFile', [DEPLOY_OPTIONS_FILE]));
        }
        const hookResults = await SfHook.run(this.config, 'sf:deploy', options);
        this.checkForHookFailures(hookResults);
        let deployers = hookResults.successes.flatMap((s) => s.result);
        if (deployers.length === 0) {
            this.log(messages.getMessage('FoundNothingToDeploy'));
        }
        else {
            if (flags.interactive) {
                deployers = await this.selectDeployers(deployers);
            }
            else {
                deployers = deployers.filter((d) => !!options[d.getName()]);
            }
            if (deployers.length === 0) {
                this.log(messages.getMessage('NothingSelectedToDeploy'));
            }
            const deployOptions = {};
            for (const deployer of deployers) {
                const opts = options[deployer.getName()] ?? {};
                // setup must be done sequentially
                // eslint-disable-next-line no-await-in-loop
                deployOptions[deployer.getName()] = await deployer.setup(flags, opts);
            }
            if (flags.interactive && (await this.askToSave())) {
                await writeJson(DEPLOY_OPTIONS_FILE, deployOptions);
                this.log();
                this.log(messages.getMessage('DeployOptionsSavedToFile', [DEPLOY_OPTIONS_FILE]));
                if (await this.shouldCommit()) {
                    await this.commit();
                    this.log(messages.getMessage('DeployOptionsIncludedInGitIgnore', [DEPLOY_OPTIONS_FILE]));
                }
            }
            const deployResults = [];
            for (const deployer of deployers) {
                // deployments must be done sequentially?
                // eslint-disable-next-line no-await-in-loop
                deployResults.push([deployer, await deployer.deploy()]);
            }
            if (deployResults.some(([, result]) => !!result && result.exitCode !== 0)) {
                process.exitCode = 1;
                this.warn(messages.getMessage('DeployersHaveNonZeroExitCode'));
                deployResults
                    .filter(([, result]) => !!result && result.exitCode !== 0)
                    .forEach(([deployer, result]) => {
                    this.log(messages.getMessage('DeployerExitCode', [deployer.getName(), result ? result.exitCode : 'unknown']));
                });
            }
        }
    }
    /**
     * If the deploy file exists, we do not want the command to be interactive. But if the file
     * does not exist then we want to force the command to be interactive.
     */
    // this used to be async when it was using fs-extra.  Presered public api
    // eslint-disable-next-line @typescript-eslint/require-await
    async isInteractive(interactive) {
        if (interactive)
            return true;
        const deployFileExists = existsSync(DEPLOY_OPTIONS_FILE);
        return deployFileExists ? false : true;
    }
    async readOptions() {
        if (existsSync(DEPLOY_OPTIONS_FILE)) {
            return parseJsonMap(await readFile(DEPLOY_OPTIONS_FILE, 'utf8'));
        }
        else {
            return {};
        }
    }
    async commit() {
        const gitignore = await readFile('.gitignore', 'utf-8');
        if (!gitignore.includes(DEPLOY_OPTIONS_FILE)) {
            const addition = `${EOL}${EOL}# Deploy Options${EOL}${DEPLOY_OPTIONS_FILE}${EOL}`;
            await writeFile('.gitignore', `${gitignore}${addition}`);
        }
        exec('git add .gitignore');
        exec(`git commit -am "Add ${DEPLOY_OPTIONS_FILE} to .gitignore"`);
    }
    // this used to be async when it was using fs-extra.  Presered public api
    // eslint-disable-next-line @typescript-eslint/require-await
    async shouldCommit() {
        return existsSync('.git') && existsSync('functions');
    }
    async askToSave() {
        const prompter = new Prompter();
        const { save } = await prompter.prompt({
            name: 'save',
            message: 'Would you like to save these deploy options for future runs?',
            type: 'confirm',
        });
        return save;
    }
    async selectDeployers(deployers) {
        const deployables = deployers.reduce((x, y) => x.concat(y.deployables), []);
        const columns = { name: 'APP OR PACKAGE', type: 'TYPE', path: 'PATH' };
        const options = deployables.map((deployable) => ({
            name: deployable.getName(),
            type: deployable.getType(),
            path: deployable.getPath(),
            value: deployable,
        }));
        const prompter = new Prompter();
        const responses = await prompter.prompt([
            {
                name: 'deployables',
                message: 'Select apps and packages to deploy:',
                type: 'checkbox',
                choices: generateTableChoices(columns, options),
            },
        ]);
        const chosenDeployers = new Map();
        for (const deployable of responses.deployables) {
            const parent = deployable.getParent();
            if (chosenDeployers.has(parent)) {
                const existing = chosenDeployers.get(parent) ?? [];
                chosenDeployers.set(parent, [...existing, deployable]);
            }
            else {
                chosenDeployers.set(parent, [deployable]);
            }
        }
        const final = [];
        for (const [parent, children] of Array.from(chosenDeployers.entries())) {
            parent.selectDeployables(children);
            final.push(parent);
        }
        return final;
    }
    checkForHookFailures(hookResults) {
        if (hookResults.failures?.length) {
            // display a table of the errors encountered; Plugin Name, Error Message
            const columns = {
                errorName: { header: 'Error Name' },
                errorMessage: { header: 'Error Message' },
            };
            const failureData = hookResults.failures.map((failure) => ({
                errorName: failure.error.name,
                errorMessage: failure.error.message,
            }));
            this.styledHeader(messages.getMessage('error.initialization.title'));
            this.table(failureData, columns, { 'no-truncate': true });
            const err = messages.createError('error.initialization');
            err.data = hookResults.failures;
            throw err;
        }
    }
}
export const writeJson = async (filename, data) => writeFile(filename, JSON.stringify(data, null, 2));
//# sourceMappingURL=deploy.js.map