/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
import { Global, TTLConfig } from '@salesforce/core';
import { Duration } from '@salesforce/kit';
import { cacheMessages } from './deploy.js';
import { maybeDestroyManifest } from './manifestCache.js';
export class DeployCache extends TTLConfig {
    static getFileName() {
        return 'deploy-cache.json';
    }
    static getDefaultOptions() {
        return {
            isGlobal: true,
            isState: true,
            filename: DeployCache.getFileName(),
            stateFolder: Global.SF_STATE_FOLDER,
            ttl: Duration.days(3),
        };
    }
    /**
     *
     * @param key jobId
     * @param value a DeployOptions object (wait is a duration, can use non-manifest options)
     * @param manifestFilePath the path to the manifest file generated by the deploy
     */
    static async set(key, value) {
        const cache = await DeployCache.create();
        // remove properties we won't cache or that are not primitives
        const { 'metadata-dir': mdDir, 'source-dir': sourceDir, wait, ...cleanValue } = value;
        cache.set(key, { ...cleanValue, wait: wait?.minutes ?? 33, isMdapi: Boolean(mdDir) });
        await cache.write();
    }
    static async unset(key) {
        const cache = await DeployCache.create();
        cache.unset(key);
        await Promise.all([cache.write(), maybeDestroyManifest(key)]);
    }
    static async update(key, obj) {
        const cache = await DeployCache.create();
        cache.update(key, obj);
        await cache.write();
    }
    resolveLatest(useMostRecent, key, throwOnNotFound = true) {
        const keyFromLatest = useMostRecent ? this.getLatestKey() : key;
        if (!keyFromLatest)
            throw cacheMessages.createError('error.NoRecentJobId');
        const jobId = this.resolveLongId(keyFromLatest);
        if (throwOnNotFound && !this.has(jobId)) {
            throw cacheMessages.createError('error.InvalidJobId', [jobId]);
        }
        return jobId;
    }
    resolveLongId(jobId) {
        if (jobId.length === 18) {
            return jobId;
        }
        else if (jobId.length === 15) {
            const match = this.keys().find((k) => k.startsWith(jobId));
            if (match) {
                return match;
            }
            throw cacheMessages.createError('error.InvalidJobId', [jobId]);
        }
        else {
            throw cacheMessages.createError('error.InvalidJobId', [jobId]);
        }
    }
    get(jobId) {
        return super.get(this.resolveLongId(jobId));
    }
}
//# sourceMappingURL=deployCache.js.map