/*
 * Copyright (c) 2021, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
/* eslint-disable class-methods-use-this */
import { EOL } from 'node:os';
import chalk from 'chalk';
import { Duration } from '@salesforce/kit';
import { AuthInfo, ConfigAggregator, StateAggregator, Messages, OrgConfigProperties, Org, } from '@salesforce/core';
import { Deployable, Deployer, generateTableChoices } from '@salesforce/sf-plugins-core';
import { DeployResultFormatter } from '../formatters/deployResultFormatter.js';
import { TestLevel } from './types.js';
import { DeployProgress } from './progressBar.js';
import { determineExitCode, executeDeploy, resolveApi } from './deploy.js';
Messages.importMessagesDirectoryFromMetaUrl(import.meta.url);
const messages = Messages.loadMessages('@salesforce/plugin-deploy-retrieve', 'deploy');
const compareOrgs = (a, b) => {
    // scratch orgs before other orgs
    if (a.isScratchOrg && !b.isScratchOrg) {
        // all scratch orgs come before non-scratch orgs
        return -1;
    }
    else {
        // sort scratch orgs by timestamp - descending
        if (a.isScratchOrg && b.isScratchOrg) {
            const aTimestamp = new Date(a.timestamp);
            const bTimestamp = new Date(b.timestamp);
            return bTimestamp.getTime() - aTimestamp.getTime();
        }
        // dev hubs after scratch but before remaining orgs
        if (a.isDevHub && !b.isScratchOrg && !b.isDevHub) {
            return -1;
        }
        // not a scratch org and not a devhub means "other" sorts last
        if (!a.isDevHub) {
            return 1;
        }
    }
    // orgs are equal by type - sort by name ascending
    return a.username.localeCompare(b.username);
};
export class DeployablePackage extends Deployable {
    pkg;
    parent;
    constructor(pkg, parent) {
        super();
        this.pkg = pkg;
        this.parent = parent;
    }
    getName() {
        return this.pkg.name;
    }
    getType() {
        return 'Salesforce App';
    }
    getPath() {
        return this.pkg.path;
    }
    getParent() {
        return this.parent;
    }
}
export class MetadataDeployer extends Deployer {
    packages;
    static NAME = 'Salesforce Apps';
    testLevel = TestLevel.NoTestRun;
    username;
    constructor(packages) {
        super();
        this.packages = packages;
        this.deployables = this.packages.map((pkg) => new DeployablePackage(pkg, this));
    }
    getName() {
        return MetadataDeployer.NAME;
    }
    async setup(flags, options) {
        if (flags.interactive) {
            this.testLevel = await this.promptForTestLevel();
            this.username = await this.promptForUsername();
        }
        else {
            if (options.directories?.length) {
                const directories = options.directories || [];
                const selected = this.deployables.filter((d) => directories.includes(d.getPath()));
                this.selectDeployables(selected);
            }
            this.testLevel = options.testLevel ?? (await this.promptForTestLevel());
            this.username = options.username ?? (await this.promptForUsername());
        }
        return {
            testLevel: this.testLevel,
            username: this.username,
            apps: this.deployables.map((d) => d.getPath()),
        };
    }
    async deploy() {
        const directories = this.deployables.map((d) => d.pkg.fullPath);
        const name = this.deployables.map((p) => chalk.cyan.bold(p.getPath())).join(', ');
        const api = await resolveApi();
        this.log(`${EOL}Deploying ${name} to ${this.username} using ${api} API`);
        const { deploy } = await executeDeploy({
            'target-org': this.username,
            'source-dir': directories,
            'test-level': this.testLevel,
            api,
        }, undefined, undefined, undefined, true);
        new DeployProgress(deploy).start();
        const result = await deploy.pollStatus(500, Duration.minutes(33).seconds);
        const formatter = new DeployResultFormatter(result, {
            'test-level': this.testLevel,
            verbose: false,
            concise: false,
            'target-org': await Org.create({ aliasOrUsername: this.username }),
        });
        formatter.display();
        const deployerResult = {
            exitCode: determineExitCode(result),
        };
        return deployerResult;
    }
    async promptForUsername() {
        const aliasOrUsername = ConfigAggregator.getValue(OrgConfigProperties.TARGET_ORG)?.value;
        const stateAggregator = await StateAggregator.getInstance();
        await stateAggregator.orgs.readAll();
        const allAliases = stateAggregator.aliases.getAll();
        let targetOrgAuth;
        // make sure the "target-org" can be used in this deploy
        if (aliasOrUsername) {
            targetOrgAuth = (await AuthInfo.listAllAuthorizations((a) => (a.username === aliasOrUsername || a.aliases?.some((alias) => alias === aliasOrUsername)) ?? false)).find((a) => a);
            if (targetOrgAuth) {
                if (targetOrgAuth?.isExpired) {
                    const continueAnswer = await this.prompt([
                        {
                            name: 'continue',
                            type: 'confirm',
                            message: chalk.red(messages.getMessage('warning.TargetOrgIsExpired', [aliasOrUsername])),
                        },
                    ]);
                    if (!continueAnswer.continue) {
                        throw messages.createError('error.UserTerminatedDeployForExpiredOrg');
                    }
                }
                else {
                    return stateAggregator.aliases.resolveUsername(aliasOrUsername);
                }
            }
        }
        if (!aliasOrUsername || targetOrgAuth?.isExpired) {
            const promises = (await AuthInfo.listAllAuthorizations((orgAuth) => !orgAuth.error && orgAuth.isExpired !== true)).map(async (orgAuth) => {
                const stat = await stateAggregator.orgs.stat(orgAuth.username);
                const timestamp = stat ? new Date(stat.mtimeMs) : new Date();
                return { ...orgAuth, timestamp };
            });
            const authorizations = await Promise.all(promises);
            if (authorizations.length > 0) {
                const newestAuths = authorizations.sort(compareOrgs);
                const options = newestAuths.map((auth) => ({
                    name: auth.username,
                    aliases: Object.entries(allAliases)
                        .filter(([, usernameOrAlias]) => usernameOrAlias === auth.username)
                        .map(([alias]) => alias)
                        .join(', '),
                    isScratchOrg: auth.isScratchOrg ? 'Yes' : 'No',
                    value: auth.username,
                }));
                const columns = { name: 'Org', aliases: 'Aliases', isScratchOrg: 'Scratch Org' };
                const { username } = await this.prompt([
                    {
                        name: 'username',
                        message: 'Select the org you want to deploy to:',
                        type: 'list',
                        choices: generateTableChoices(columns, options, false),
                    },
                ]);
                if (targetOrgAuth?.isExpired) {
                    const setTargetOrg = await this.prompt([
                        {
                            name: 'save',
                            type: 'confirm',
                            message: messages.getMessage('save.as.default', [username]),
                        },
                    ]);
                    if (setTargetOrg.save) {
                        const authInfo = await AuthInfo.create({ username });
                        await authInfo.setAsDefault({ org: true });
                    }
                }
                return username;
            }
            else {
                throw messages.createError('errors.NoOrgsToSelect');
            }
        }
        throw new Error('Unexpected: You should not have arrived here.');
    }
    async promptForTestLevel() {
        const { testLevel } = await this.prompt([
            {
                name: 'testLevel',
                message: 'Select the test level you would like to run:',
                type: 'list',
                loop: false,
                pageSize: 4,
                choices: [
                    { name: "Don't run tests", value: TestLevel.NoTestRun, short: "Don't run tests" },
                    { name: 'Run local tests', value: TestLevel.RunLocalTests, short: 'Run local tests' },
                    {
                        name: 'Run all tests in environment',
                        value: TestLevel.RunAllTestsInOrg,
                        short: 'Run all tests in environment',
                    },
                ],
            },
        ]);
        return testLevel;
    }
}
//# sourceMappingURL=metadataDeployer.js.map