/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
import { strict as assert } from 'node:assert';
import { SfCommand, Flags } from '@salesforce/sf-plugins-core';
import { Messages, scratchOrgResume, ScratchOrgCache, Lifecycle, scratchOrgLifecycleEventName, } from '@salesforce/core';
import { buildStatus } from '../../../shared/scratchOrgOutput.js';
Messages.importMessagesDirectoryFromMetaUrl(import.meta.url);
const messages = Messages.loadMessages('@salesforce/plugin-org', 'resume_scratch');
export default class EnvResumeScratch extends SfCommand {
    static summary = messages.getMessage('summary');
    static description = messages.getMessage('description');
    static examples = messages.getMessages('examples');
    static aliases = ['env:resume:scratch'];
    static deprecateAliases = true;
    static flags = {
        'job-id': Flags.salesforceId({
            char: 'i',
            length: 'both',
            summary: messages.getMessage('flags.job-id.summary'),
            description: messages.getMessage('flags.job-id.description'),
            exactlyOne: ['use-most-recent', 'job-id'],
            startsWith: '2SR',
        }),
        'use-most-recent': Flags.boolean({
            char: 'r',
            summary: messages.getMessage('flags.use-most-recent.summary'),
            exactlyOne: ['use-most-recent', 'job-id'],
        }),
    };
    async run() {
        const { flags } = await this.parse(EnvResumeScratch);
        const cache = await ScratchOrgCache.create();
        const lifecycle = Lifecycle.getInstance();
        const jobId = flags['use-most-recent'] ? cache.getLatestKey() : flags['job-id'];
        if (!jobId && flags['use-most-recent'])
            throw messages.createError('error.NoRecentJobId');
        // oclif doesn't know that the exactlyOne flag will ensure that one of these is set, and there we definitely have a jobID.
        assert(jobId);
        const { hubBaseUrl } = cache.get(jobId);
        let lastStatus;
        lifecycle.on(scratchOrgLifecycleEventName, async (data) => {
            lastStatus = buildStatus(data, hubBaseUrl);
            this.spinner.status = lastStatus;
            return Promise.resolve();
        });
        this.log();
        this.spinner.start('Creating Scratch Org');
        const { username, scratchOrgInfo, authFields, warnings } = await scratchOrgResume(jobId);
        this.spinner.stop(lastStatus);
        this.log();
        this.logSuccess(messages.getMessage('success'));
        return { username, scratchOrgInfo, authFields, warnings, orgId: authFields?.orgId };
    }
}
//# sourceMappingURL=scratch.js.map